/**
 * OW2 Util
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 *
 * --------------------------------------------------------------------------
 * $Id: WebservicesDeploymentDescLoader.java 4689 2009-02-23 14:23:05Z sauthieg $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.ee.metadata.ws.impl.xml.parsing;

import org.ow2.util.ee.metadata.common.impl.xml.parsing.CommonDeploymentDescLoader;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException;
import org.ow2.util.ee.metadata.common.impl.xml.parsing.HandlerChainsLoader;
import org.ow2.util.ee.metadata.ws.impl.xml.struct.PortComponent;
import org.ow2.util.ee.metadata.ws.impl.xml.struct.WebserviceDescription;
import org.ow2.util.ee.metadata.ws.impl.xml.struct.Webservices;
import org.ow2.util.ee.metadata.ws.impl.xml.struct.ServiceImplBean;
import org.ow2.util.log.Log;
import org.ow2.util.log.LogFactory;
import org.ow2.util.xml.DocumentParser;
import org.ow2.util.xml.DocumentParserException;
import org.ow2.util.xml.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import javax.xml.namespace.QName;
import java.net.URL;
import java.util.Hashtable;
import java.util.Map;

/**
 * The WebservicesDeploymentDescLoader loads the webservices.xml file.
 *
 * @author Guillaume Sauthier
 */
public class WebservicesDeploymentDescLoader extends CommonDeploymentDescLoader {

    /**
     * Logger.
     */
    private static Log logger = LogFactory.getLog(WebservicesDeploymentDescLoader.class);

    /**
     * Standard JAX-WS binding's aliases.
     */
    private static Map<String, String> bindingAliases;

    // Fill the aliases Map
    static {
        bindingAliases = new Hashtable<String, String>();
        bindingAliases.put("##SOAP11_HTTP", "http://schemas.xmlsoap.org/wsdl/soap/http");
        bindingAliases.put("##SOAP11_HTTP_MTOM", "http://schemas.xmlsoap.org/wsdl/soap/http?mtom=true");
        bindingAliases.put("##SOAP12_HTTP", "http://www.w3.org/2003/05/soap/bindings/HTTP/");
        bindingAliases.put("##SOAP12_HTTP_MTOM", "http://www.w3.org/2003/05/soap/bindings/HTTP/?mtom=true");
        bindingAliases.put("##XML_HTTP", "http://www.w3.org/2004/08/wsdl/http");
    }

    /**
     * The loaded webservices element.
     */
    private Webservices webservices;

    /**
     * Utility class, no public constructor.
     *
     * @param url the url that is pointing to the Xml file.
     */
    private WebservicesDeploymentDescLoader(final URL url) {
        super(url);
        this.webservices = new Webservices();
    }

    /**
     * Load the ejb-jar.xml file.
     * @param url the URL of the XML file.
     * @throws org.ow2.util.ee.metadata.common.impl.xml.parsing.ParsingException if parsing of XML file fails.
     * @return an application object.
     */
    public static Webservices loadDeploymentDescriptor(final URL url) throws ParsingException {
        logger.debug("Analyzing url {0}", url);

        WebservicesDeploymentDescLoader wsLoader = new WebservicesDeploymentDescLoader(url);
        wsLoader.parse();
        return wsLoader.getWebservices();
    }

    /**
     * Analyze the URL and build an EJB3 object which contains data of the XML.
     * @throws ParsingException if the analyze of the file fails.
     */
    public void parse() throws ParsingException {
        // Get document
        Document document = null;
        try {
            document = DocumentParser.getDocument(getURL(),
                                                  isValidating(),
                                                  new WebservicesEntityResolver());
        } catch (DocumentParserException e) {
            throw new ParsingException("Cannot parse the url", e);
        }

        // Root element = <webservices>
        Element webservicesRootElement = document.getDocumentElement();

        // Define NS
        defineXMLNS(webservicesRootElement);

        // @version
        String version = webservicesRootElement.getAttribute("version");
        webservices.setVersion(version);

        // webservice-description
        NodeList wsDescriptionsList = webservicesRootElement.getElementsByTagNameNS(getXMLNS(),
                                                                                    WebserviceDescription.NAME);

        for (int i = 0; i < wsDescriptionsList.getLength(); i++) {
            Element wsDescriptionElement = (Element) wsDescriptionsList.item(i);

            // Create struct
            WebserviceDescription wd = new WebserviceDescription();
            webservices.addWebserviceDescription(wd);

            // analyze the webservice-description element
            analyzeWebserviceDescription(wsDescriptionElement, wd);
        }
    }

    /**
     * Analyze the webservice-description element.
     * @param wsDescriptionElement webservice-description element
     * @param webserviceDescription mapped object to be filled.
     */
    private void analyzeWebserviceDescription(Element wsDescriptionElement, WebserviceDescription webserviceDescription) {

        // webservice-description-name
        String name = XMLUtils.getStringValueElement(getXMLNS(), wsDescriptionElement, "webservice-description-name");
        webserviceDescription.setName(name);

        // wsdl-file
        String wsdlFile = XMLUtils.getStringValueElement(getXMLNS(), wsDescriptionElement, "wsdl-file");
        webserviceDescription.setWsdlFile(wsdlFile);

        // port-component
        NodeList portComponentList = wsDescriptionElement.getElementsByTagNameNS(getXMLNS(),
                                                                                 PortComponent.NAME);

        for (int i = 0; i < portComponentList.getLength(); i++) {
            Element portComponentElement = (Element) portComponentList.item(i);

            // Create struct
            PortComponent pc = new PortComponent(webserviceDescription);
            webserviceDescription.addPortComponent(pc);

            // analyze the port-component element
            analyzePortComponent(portComponentElement, pc);
        }

    }

    /**
     * Analyze the port-component element.
     * @param portComponentElement port-component element
     * @param portComponent mapped object to be filled.
     */
    private void analyzePortComponent(Element portComponentElement, PortComponent portComponent) {

        // port-component-name
        String name = XMLUtils.getStringValueElement(getXMLNS(), portComponentElement, "port-component-name");
        portComponent.setName(name);

        // wsdl-service
        NodeList wsdlServiceNodes = portComponentElement.getElementsByTagNameNS(getXMLNS(), "wsdl-service");
        if (wsdlServiceNodes.getLength() == 1) {
            Element wsdlServiceNode = (Element) wsdlServiceNodes.item(0);
            QName wsdlService = XMLUtils.getValueAsQName(wsdlServiceNode);
            portComponent.setWsdlService(wsdlService);
        }

        // wsdl-port
        NodeList wsdlPortNodes = portComponentElement.getElementsByTagNameNS(getXMLNS(), "wsdl-port");
        if (wsdlPortNodes.getLength() == 1) {
            Element wsdlPortNode = (Element) wsdlPortNodes.item(0);
            QName wsdlPort = XMLUtils.getValueAsQName(wsdlPortNode);
            portComponent.setWsdlPort(wsdlPort);
        }

        // enable-mtom
        String enableMtom = XMLUtils.getStringValueElement(getXMLNS(), portComponentElement, "enable-mtom");
        if ("true".equals(enableMtom)) {
            // MTOM is only activated when 'true' is provided, false in all other cases
            portComponent.setMTOMEnabled(true);
        }

        // protocol-binding
        String protocolBinding = XMLUtils.getStringValueElement(getXMLNS(), portComponentElement, "protocol-binding");
        if (protocolBinding != null) {
            // Replace known aliases with their real URI values
            String aliased = bindingAliases.get(protocolBinding);
            // If no mapping was found, use the binding as-is
            if (aliased == null) {
                aliased = protocolBinding;
            }
            portComponent.setProtocolBinding(aliased);
        }

        // service-endpoint-interface
        String sei = XMLUtils.getStringValueElement(getXMLNS(), portComponentElement, "service-endpoint-interface");
        portComponent.setServiceEndpointInterface(sei);

        // service-impl-bean
        NodeList sibNodes = portComponentElement.getElementsByTagNameNS(getXMLNS(), "service-impl-bean");
        if (sibNodes.getLength() == 1) {
            Element sibNode = (Element) sibNodes.item(0);

            // Create struct
            ServiceImplBean sib = new ServiceImplBean();
            portComponent.setServiceImplBean(sib);

            // analyze the service-impl-bean element
            analyzeServiceImplBean(sibNode, sib);


        }

        // handler-chains
        NodeList hcNodes = portComponentElement.getElementsByTagNameNS(getXMLNS(), "handler-chains");
        if (hcNodes.getLength() == 1) {
            Element handlerChainsElement = (Element) hcNodes.item(0);
            try {
                portComponent.setHandlerChains(HandlerChainsLoader.loadHandlerChains(handlerChainsElement));
            } catch (ParsingException e) {
                logger.debug("Should never occurs, the element is already parsed", e);
            }
        }

    }

    /**
     * Analyze the service-impl-bean element
     * @param sibElement SIB element
     * @param sib mapped object to be filled
     */
    private void analyzeServiceImplBean(Element sibElement, ServiceImplBean sib) {

        // servlet-link
        String servletLink = XMLUtils.getStringValueElement(getXMLNS(), sibElement, "servlet-link");
        sib.setServletLink(servletLink);

        // ejb-link
        String ejbLink = XMLUtils.getStringValueElement(getXMLNS(), sibElement, "ejb-link");
        sib.setEjbLink(ejbLink);

    }

    /**
     * @return the loaded instance
     */
    public Webservices getWebservices() {
        return webservices;
    }
}
